@@ -31,8 +31,9 @@
 #include "cintltst.h"
 #include "ubrkimpl.h"
 #include "toolutil.h" /* for uprv_fileExists() */
-#include <stdlib.h>
+#include <stdbool.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* includes for TestSwapData() */
 #include "udataswp.h"
@@ -45,6 +46,7 @@
 #include "ucol_imp.h"
 #include "ucol_swp.h"
 #include "ucnv_bld.h"
+#include "udataswp.h"
 #include "sprpimpl.h"
 #include "rbbidata.h"
 
@@ -116,7 +118,7 @@ static void lots_of_mallocs()
 #endif
 
 #if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
-static void TestUDataOpen(){
+static void TestUDataOpen(void){
     UDataMemory *result;
     UErrorCode status=U_ZERO_ERROR;
     const char* memMap[][2]={
@@ -441,7 +443,7 @@ static const ICU_COMMON_Data_Header gEmptyHeader = {
 };
 
 
-static void TestUDataSetAppData(){
+static void TestUDataSetAppData(void){
 /*    UDataMemory      *dataItem;*/
 
     UErrorCode        status=U_ZERO_ERROR;
@@ -496,7 +498,7 @@ static void TestUDataSetAppData(){
         dataItem = udata_open("appData1", "res", "te_IN", &status); **/
 }
 
-static char *safeGetICUDataDirectory() {
+static char *safeGetICUDataDirectory(void) {
     const char *dataDir = u_getDataDirectory();  /* Returned string vanashes with u_cleanup */
     char *retStr = NULL;
     if (dataDir != NULL) {
@@ -506,7 +508,7 @@ static char *safeGetICUDataDirectory() {
     return retStr;
 }
     
-static void TestUDataFileAccess(){
+static void TestUDataFileAccess(void){
     UErrorCode status;
     char            *icuDataDir;
     icuDataDir = safeGetICUDataDirectory();   /* save icu data dir, so we can put it back
@@ -558,6 +560,7 @@ static UBool U_CALLCONV
 isAcceptable1(void *context,
              const char *type, const char *name,
              const UDataInfo *pInfo) {
+    (void)context; // suppress compiler warnings about unused variable
 
     if( pInfo->size>=20 &&
         pInfo->isBigEndian==U_IS_BIG_ENDIAN &&
@@ -568,10 +571,10 @@ isAcceptable1(void *context,
         pInfo->dataFormat[3]==0x6c &&
         pInfo->formatVersion[0]==3 )
     {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable1()\n", name, type);
-        return TRUE;
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable1()\n", name, type);
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable1():-\n"
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable1():-\n"
             "\tsize              = %d\n"
             "\tisBigEndian       = %d\n"
             "\tcharsetFamily     = %d\n"
@@ -581,8 +584,8 @@ isAcceptable1(void *context,
             name, type, pInfo->size,  pInfo->isBigEndian, pInfo->charsetFamily, pInfo->formatVersion[0], 
             pInfo->dataVersion[0], pInfo->dataFormat[0], pInfo->dataFormat[1], pInfo->dataFormat[2], 
             pInfo->dataFormat[3]);  
-        log_verbose("Call another verifing function to accept the data\n");
-        return FALSE;
+        log_verbose("Call another verifying function to accept the data\n");
+        return false;
     }
 }
 
@@ -590,6 +593,7 @@ static UBool U_CALLCONV
 isAcceptable2(void *context, 
              const char *type, const char *name,
       const UDataInfo *pInfo){
+    (void)context; // suppress compiler warnings about unused variable
     UVersionInfo unicodeVersion;
 
     u_getUnicodeVersion(unicodeVersion);
@@ -604,12 +608,12 @@ isAcceptable2(void *context,
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==unicodeVersion[0] )
     {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable2()\n", name, type);
-        return TRUE;
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable2()\n", name, type);
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable2()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable2()\n", name, type);
 
-        return FALSE;
+        return false;
     }
 
 
@@ -618,6 +622,7 @@ static UBool U_CALLCONV
 isAcceptable3(void *context, 
              const char *type, const char *name,
              const UDataInfo *pInfo){
+    (void)context; // suppress compiler warnings about unused variable
 
     if( pInfo->size>=20 &&
         pInfo->isBigEndian==U_IS_BIG_ENDIAN &&
@@ -628,19 +633,19 @@ isAcceptable3(void *context,
         pInfo->dataFormat[3]==0x74 &&
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==1   ) {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable3()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable3()\n", name, type);
 
-        return TRUE;
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable3()\n", name, type);
-        return FALSE;
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable3()\n", name, type);
+        return false;
     }
 
 
 }
 
 #if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
-static void TestUDataOpenChoiceDemo1() {
+static void TestUDataOpenChoiceDemo1(void) {
     UDataMemory *result;
     UErrorCode status=U_ZERO_ERROR;
 
@@ -728,19 +733,19 @@ isAcceptable(void *context,
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==1   &&
         *((int*)context) == 2 ) {
-        log_verbose("The data from\"%s.%s\" IS acceptable using the verifing function isAcceptable()\n", name, type);
+        log_verbose("The data from\"%s.%s\" IS acceptable using the verifying function isAcceptable()\n", name, type);
 
-        return TRUE;
+        return true;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable()\n", name, type);
-        return FALSE;
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable()\n", name, type);
+        return false;
     }
 }
 
 
 /* This test checks to see if the isAcceptable function is being called correctly. */
 
-static void TestUDataOpenChoiceDemo2() {
+static void TestUDataOpenChoiceDemo2(void) {
     UDataMemory *result;
     UErrorCode status=U_ZERO_ERROR;
     int i;
@@ -787,7 +792,7 @@ static void TestUDataOpenChoiceDemo2() {
     }
 }
 
-static void TestUDataGetInfo() {
+static void TestUDataGetInfo(void) {
 
     UDataMemory *result;
     /* UDataInfo cf. udata.h */
@@ -864,7 +869,7 @@ static void TestUDataGetInfo() {
     udata_close(result);
 }
 
-static void TestUDataGetMemory() {
+static void TestUDataGetMemory(void) {
 
     UDataMemory *result;
     const int32_t *table=NULL;
@@ -912,7 +917,7 @@ static void TestUDataGetMemory() {
 
 }
 
-static void TestErrorConditions(){
+static void TestErrorConditions(void){
 
     UDataMemory *result=NULL;
     UErrorCode status=U_ZERO_ERROR;
@@ -1029,7 +1034,7 @@ static void TestErrorConditions(){
 }
 
 /* Test whether apps and ICU can each have their own root.res */
-static void TestAppData()
+static void TestAppData(void)
 {
     UResourceBundle *icu, *app;
     UResourceBundle *tmp = NULL;
@@ -1117,7 +1122,7 @@ static void TestAppData()
 }
 #endif
 
-static void TestICUDataName()
+static void TestICUDataName(void)
 {
     UVersionInfo icuVersion;
     char expectDataName[20];
@@ -1143,7 +1148,7 @@ static void TestICUDataName()
                 typeChar = 'l';
                 break;
           default:
-                log_err("Expected 1 or 0 for U_IS_BIG_ENDIAN, got %d!\n", (int)U_IS_BIG_ENDIAN);
+                log_err("Expected 1 or 0 for U_IS_BIG_ENDIAN, got %d!\n", U_IS_BIG_ENDIAN);
                 /* return; */
           }
           break;
@@ -1153,7 +1158,7 @@ static void TestICUDataName()
     }
 
     /* Only major number is needed. */
-    sprintf(expectDataName, "%s%d%c",
+    snprintf(expectDataName, sizeof(expectDataName), "%s%d%c",
                 "icudt",
                 (int)icuVersion[0],
                 typeChar);
@@ -1233,7 +1238,8 @@ static const struct {
 
     {0x31, 0x31, 0x31, 0x31},     /* dataFormat="1111" */
     {0, 0, 0, 0},                 /* formatVersion */
-    {0, 0, 0, 0}}                 /* dataVersion */
+    {0, 0, 0, 0}},                /* dataVersion */
+    0
 };
 #endif
 
@@ -1328,7 +1334,7 @@ static const struct {
     /* EUC-TW (3-byte) conversion table file without extension */
     {"ibm-964_P110-1999",        "cnv", ucnv_swap},
     /* GB 18030 (4-byte) conversion table file without extension */
-    {"gb18030",                  "cnv", ucnv_swap},
+    {"gb18030-2022",             "cnv", ucnv_swap},
     /* MBCS conversion table file with extension */
     {"*test4x",                  "cnv", ucnv_swap},
     /*
@@ -1349,7 +1355,7 @@ static const struct {
 
 #if !UCONFIG_NO_BREAK_ITERATION
     {"char",                     "brk", ubrk_swap},
-    {"thaidict",                 "dict",udict_swap},
+    {"laodict",                  "dict",udict_swap},
 #endif
 
 #if 0
@@ -1395,6 +1401,7 @@ static const struct {
 
 static void U_CALLCONV
 printError(void *context, const char *fmt, va_list args) {
+    (void)context; // suppress compiler warnings about unused variable
     vlog_info("[swap] ", fmt, args);
     log_err("\n");  /* Register error */
 }
@@ -1620,12 +1627,12 @@ TestSwapCase(UDataMemory *pData, const char *name,
 
 static void U_CALLCONV
 printErrorToString(void *context, const char *fmt, va_list args) {
-    vsprintf((char *)context, fmt, args);
+    vsnprintf((char *)context, 100, fmt, args);
 }
 
 #if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
 static void
-TestSwapData() {
+TestSwapData(void) {
     char name[100];
     UDataSwapper *ds;
     UDataMemory *pData;
@@ -1741,7 +1748,7 @@ TestSwapData() {
 }
 #endif
 
-static void PointerTableOfContents() {
+static void PointerTableOfContents(void) {
     UDataMemory      *dataItem;
     UErrorCode        status=U_ZERO_ERROR;
        
@@ -1852,7 +1859,7 @@ static void TestTZDataDir(void) {
     //     whatever/.../testdata/out/testdata
     // The test data puts an old (2014a) version of the time zone data there.
 
-    // Switch ICU to the testdata version of zoneinfo64.res, which is verison 2014a.
+    // Switch ICU to the testdata version of zoneinfo64.res, which is version 2014a.
     ctest_resetICU();
     u_setTimeZoneFilesDirectory(testDataPath, &status);
     tzDataVersion = ucal_getTZDataVersion(&status);
